@@ -1,12 +1,10 @@
 from typing import Optional
 
 from pydantic.dataclasses import dataclass
-from serde import deserialize
 
 from metaphor.common.base_config import BaseConfig
 
 
-@deserialize
 @dataclass
 class DbtRunConfig(BaseConfig):
     manifest: str
